import { getConnection } from "./../database/databaseMysql"

const getCamaras = async (req, res) => {
  try {
    const connection = await getConnection()
    const result = await connection.query("SELECT id, nombre, ip, posicion_x, posicion_y, rotation FROM camaras")
    res.json(result) 
  } catch (error) {
    res.status(500)
    res.send(error.message)
  }
}

const getCamarasTable = async (req, res) => {
  try {
    // Conectamos a la DB
    const connection = await getConnection()
    // Obtenemos las camaras
    const camaras = await connection.query("SELECT id, nombre, ip, posicion_x, posicion_y, rotation, link, active FROM camaras")
    // Obtenemos las relaciones camara-grupo
    const camarasGrupos = await connection.query("SELECT camara_id, grupo_id FROM grupos_camaras WHERE camara_id IN (?)", [camaras.map(camara => camara.id)])
    // Obtenemos los grupos
    const grupoIds = camarasGrupos.map(camaraGrupo => camaraGrupo.grupo_id)
    const grupos = await connection.query("SELECT id, nombre, descripcion FROM grupos WHERE id IN (?)", [grupoIds])
    // Combinar los resultados
    const camarasWithGroups = camaras.map(camara => {
      const camaraGrupos = camarasGrupos.filter(camaraGrupo => camaraGrupo.camara_id === camara.id)
      const gruposCamaras = camaraGrupos.map(camaraGrupo => grupos.find(grupo => grupo.id === camaraGrupo.grupo_id))
      return { ...camara, grupos: gruposCamaras }
    })
    
    res.json(camarasWithGroups)
  } catch (error) {
    res.status(500)
    res.send(error.message)
  }
}

const getCamarasByGroup = async (req, res) => {
  try {
    const {group} = req.params
    const connection = await getConnection()
    const result = await connection.query(
      `SELECT camaras.* FROM camaras 
      JOIN grupos_camaras ON camaras.id = grupos_camaras.camara_id 
      JOIN grupos ON grupos.id = grupos_camaras.grupo_id 
      WHERE grupos.id = ?`, group)
    res.json(result) 
  } catch (error) {
    res.status(500)
    res.send(error.message)
  }
}

const getCamarasByGroups = async (req, res) => {
  try {
    const groups = req.query.groups.split(',') // Separar los grupos en un array
    const groupIds = groups.map(Number) // Convertir los grupos a números (opcional, dependiendo del tipo de datos)

    const connection = await getConnection()
    const results = await Promise.all(groupIds.map(groupId =>
      connection.query(
        `SELECT DISTINCT camaras.*, grupos_camaras.grupo_id FROM camaras 
        JOIN grupos_camaras ON camaras.id = grupos_camaras.camara_id 
        WHERE grupos_camaras.grupo_id=? AND camaras.active=1`, groupId)
    ))

    // Filtrar las cámaras duplicadas
    const combinedResults = results.reduce((combined, result) => {
      result.forEach((camara) => {
        // Verificar si la cámara ya existe en el array combinado
        const existingCamara = combined.find(
          (c) => c.id === camara.id && c.group_id === camara.group_id
        );

        // Si la cámara no existe, agregarla al array combinado
        if (!existingCamara) {
          combined.push(camara);
        }
      });
      return combined;
    }, []);
    res.json(combinedResults)

  } catch (error) {
    res.status(500).send(error.message)
  }
}


const getCamara = async (req, res) => {
  try {
    const {id} = req.params
    const connection = await getConnection()
    const result = await connection.query("SELECT id, nombre, ip, posicion_x, posicion_y, active  FROM camaras WHERE id = ?", id)
    res.json(result) 
  } catch (error) {
    res.status(500)
    res.send(error.message)
  }
}

const getCamaraGroups = async (req, res) => {
  try {
    const { id } = req.params;
    const connection = await getConnection();
    
    // Obtener los grupos asociados a la cámara especificada
    const query = `
      SELECT g.id, g.nombre
      FROM grupos g
      INNER JOIN grupos_camaras gc ON gc.grupo_id = g.id
      WHERE gc.camara_id = ?
    `;
    const result = await connection.query(query, [id]);
    
    res.json(result);
  } catch (error) {
    res.status(500).send(error.message);
  }
}

const addCamara = async (req, res) => {
  try {
    const { nombre, ip, posicion_x, posicion_y, active, grupos } = req.body
 
    // Validation of values 
    if( nombre === undefined || ip === undefined || posicion_x === undefined || posicion_y === undefined || active === undefined || grupos === undefined)
      res.status(400).json({message: "Bad Request. Please give all data needed."+req.body})

    const camara = { nombre, ip, posicion_x, posicion_y, active }
    const connection = await getConnection()
    const result = await connection.query("INSERT INTO camaras SET ?", camara)

    // Get the ID of the newly inserted camera
    const camaraId = result.insertId;

    // Insert the camera into the camaras_grupos table
    for (let i = 0; i < grupos.length; i++) {
      const grupo = grupos[i];
      await connection.query("INSERT INTO grupos_camaras (camara_id, grupo_id) VALUES (?, ?)", [camaraId, grupo])
    }

    res.json(result)
  } catch (error) {
    res.status(500)
    res.send(error.message)
  }
}


const deleteCamara = async (req, res) => {
  try {
    const {id} = req.params
    const connection = await getConnection()

    // Primero, eliminar las referencias de la cámara en la tabla 'grupos_camaras'
    await connection.query("DELETE FROM grupos_camaras WHERE camara_id = ?", [id])
    
    // Luego, eliminar la cámara de la tabla 'camaras'
    const result = await connection.query("DELETE FROM camaras WHERE id = ?", [id])
    
    res.json(result) 
  } catch (error) {
    res.status(500)
    res.send(error.message)
  }
}

const updateCamara = async (req, res) => {
  try {
    const { id } = req.params;
    const { nombre, ip, posicion_x, posicion_y, grupos, active } = req.body;
    
    // Validación de valores
    if (id === undefined && (nombre === undefined || ip === undefined)) {
      res.status(400).json({ message: "Bad Request. Please provide all the required data." });
    }

    const camara = { id, nombre, ip, posicion_x, posicion_y, active };
    const connection = await getConnection();

    // Actualizar la cámara en la tabla 'camaras'
    const updateResult = await connection.query("UPDATE camaras SET ? WHERE id = ?", [camara, id]);

    // Eliminar las referencias antiguas de la cámara en la tabla 'grupos_camaras'
    await connection.query("DELETE FROM grupos_camaras WHERE camara_id = ?", [id]);

    // Insertar las nuevas referencias de los grupos de la cámara en la tabla 'grupos_camaras'
    for (let i = 0; i < grupos.length; i++) {
      const grupo = grupos[i];
      await connection.query("INSERT INTO grupos_camaras (camara_id, grupo_id) VALUES (?, ?)", [id, grupo]);
    }

    res.json(updateResult);
  } catch (error) {
    res.status(500).send(error.message);
  }
};

const updatePosition = async (req, res) => {
  try {
    console.log(req.body)
    const {id} = req.params
    const { posicion_x, posicion_y, rotation } = req.body
    // Validation of values 
    if( id === undefined)
      res.status(400).json({message: "Bad Request. Please give all data needed."})
    const camara = { posicion_x, posicion_y, rotation }
    const connection = await getConnection()
    const result = await connection.query("UPDATE camaras SET ? WHERE id = ?", [camara, id])
    res.json(result) 
  } catch (error) {
    res.status(500)
    res.send(error.message)
  }
}

// USER DATA PERMISSIONS

const getUserById = async (req, res) => {
  try {
    const {id} = req.params
    const connection = await getConnection()
    const result = await connection.query("SELECT id, nombre, permisos FROM users WHERE user_id=?", id)
    res.json(result)
  } catch (error) {
    res.status(500)
    res.send(error.message)
  }
}

// GROUPS
const getGroups = async (req, res) => {
  try {
    const connection = await getConnection();
    const result = await connection.query("SELECT * FROM grupos");
    const groups = result.map(row => ({
      id: row.id,
      nombre: row.nombre,
      descripcion: row.descripcion
    }));
    res.json(groups);
  } catch (error) {
    res.status(500).send(error.message);
  }
}

const getGroup = async (req, res) => {
  try {
    const {id} = req.params
    const connection = await getConnection()
    const result = await connection.query("SELECT id, nombre, descripcion FROM grupos WHERE id=?", id)
    res.json(result[0])
  } catch (error) {
    res.status(500)
    res.send(error.message)
  }
}

const addGroup = async (req, res) => {
  try {
    const { nombre, descripcion } = req.body

    // Validación de los valores
    if (!nombre || !descripcion) {
      return res.status(400).json({ message: "Bad Request. Please provide all required data." })
    }

    const connection = await getConnection();
    const result = await connection.query("INSERT INTO grupos (nombre, descripcion) VALUES (?, ?)", [nombre, descripcion])

    res.json(result)
  } catch (error) {
    res.status(500).send(error.message)
  }
}

const deleteGroup = async (req, res) => {
  try {
    const { id } = req.params
    const connection = await getConnection()

    // Eliminar las relaciones de la tabla 'grupos_camaras' para el grupo especificado
    await connection.query("DELETE FROM grupos_camaras WHERE grupo_id = ?", [id])

    // Eliminar el grupo de la tabla 'grupos'
    const result = await connection.query("DELETE FROM grupos WHERE id = ?", [id])
    
    res.json(result) 
  } catch (error) {
    res.status(500)
    res.send(error.message)
  }
}

const updateGroup = async (req, res) => {
  try {
    const { id } = req.params
    const { nombre, descripcion } = req.body
    
    // Validación de valores
    if (nombre === undefined || descripcion === undefined) {
      res.status(400).json({ message: "Bad Request. Please provide all the required data." })
    }

    const group = { id, nombre, descripcion }
    const connection = await getConnection()

    // Actualizar el grupo en la tabla 'grupos'
    const updateResult = await connection.query("UPDATE grupos SET ? WHERE id = ?", [group, id])

    res.json(updateResult)
  } catch (error) {
    res.status(500).send(error.message)
  }
}

const getGroupsTable = async (req, res) => {
  try {
    // Conectamos a la DB
    const connection = await getConnection()

    // Obtener los grupos correspondientes
    const groups = await connection.query("SELECT id, nombre, descripcion FROM grupos")

    res.json(groups)

  } catch (error) {
    res.status(500)
    res.send(error.message)
  }
}

const getConfig = async (req, res) => {
  try {
    const { id } = req.params

    const connection = await getConnection()
    const result = await connection.query("SELECT * FROM configs WHERE id = ?", [id])
    res.json(result) 
  } catch (error) {
    res.status(500)
    res.send(error.message)
  }
}


export const methods = {
  getCamaras,
  getCamarasTable,
  getCamarasByGroup,
  getCamarasByGroups,
  getCamaraGroups,
  getCamara,
  addCamara,
  deleteCamara,
  updateCamara,
  updatePosition,
  getGroups,
  getGroup,
  deleteGroup,
  updateGroup,
  getGroupsTable,
  addGroup,
  getConfig
}